#!/usr/local/bin/ruby
=begin

=Name
mkdep - make dependency list for make.

=Synopsis
mkdep source ...

=Description
mkdep reads a specified source file and create dependency list
suitable for a Makefile entry. Dependencies are collected from
the source file. If source, say foo.c, includes foo.h, mkdep 
append it to a dependency list of foo.c. This process goes
recursively so if foo.h includes bar.h, bar.h is also appended
to the list.

=Bugs
* Hence dependency collection does not recognize compile options,
  dependency lists contain bogus entries, which is never included.
* When an included file is not available, mkdep ignores the file.
  So a resulting dependency may be not useful enough.

=end


def pathinc(fn)
  f = nil
  $ipaths.each do
    |f|
    f = File.join(f, fn)
    break if File.exists? f
    f = nil
  end
  f.sub(%=^./=, "") if f
end

def getdep(fn)
  dep = [fn]
  fn = pathinc(fn)
  begin
    File.open(fn) do
      |s|
      s.each do
	|ln|
	case ln
	when /#\s*include\s+"(.*)"/
	  f = pathinc($1)
	  dep << f
	  dep |= getdep(f)
	end
      end
    end
  rescue Errno::ENOENT
    # ignore missing includes
  end
  dep
end

$ipaths = ARGV.collect { |arg| arg =~ /^-I(.*)/; $1 }.compact
$ipaths.unshift(".")
ARGV.reject! { |arg| arg =~ /^-I/ }

$dep = {}
ARGV.each do
  |fn|
  $dep[fn] = getdep(fn)
end

$dep.each do
  |fn, dep|
  puts "#{fn.sub(/\.c$/, '.o')} : #{dep.join(' ')}"
end
